﻿using System.Collections.Concurrent;

namespace Whereis.Utils;
public class WhereisUtils {

    public string[] ExcludedDirectories = new string[] {};
    public string[] prepare() {
        if (wasExecutedWithSudo() == false) {
            ExcludedDirectories = new string[] { "/dev", "/proc", "/tmp", "/boot", "/run", "/root", "/sys", "/etc", "/var", "/lib", "/lib64", "/usr", "/sbin", "/lost+found" };
        } else {
            ExcludedDirectories = new string[] { "/dev", "/proc" };
        }
        
        Console.WriteLine("Exclude " + string.Join(",", ExcludedDirectories));
        return ExcludedDirectories;
    }

    private bool wasExecutedWithSudo() {
        bool sudo = false;
        try {
            string protectedDirectoryPath = "/root";

            // Attempt to open the protected directory
            Directory.GetFiles(protectedDirectoryPath);

            // Access succeeded
            Console.WriteLine("Program executed with sudo privileges.");
            sudo = true;
        }
        catch (UnauthorizedAccessException) {
            // Access denied
            Console.WriteLine("Program executed without sudo privileges.");
        }

        return sudo;
    }
    
    public ConcurrentBag<string> FindFiles(string fileName, string fileExtension, string searchPath, int depth = 0, int maxDepth = 5) {        
        var foundFiles = new ConcurrentBag<string>();

        if (depth > maxDepth) {
            return foundFiles;
        }

        try {
            Parallel.ForEach(Directory.EnumerateDirectories(searchPath), (subdirectory) => {
                try {
                    // Skip excluded directories
                    if (IsExcludedDirectory(subdirectory)) {
                        return;
                    }

                    foreach (string filePath in FindFiles(fileName, fileExtension, subdirectory, depth + 1, maxDepth)) {
                        foundFiles.Add(filePath);
                    }
                }
                catch (UnauthorizedAccessException) {
                    // Handle unauthorized access exception for the subdirectory
                    Console.WriteLine($"Unauthorized access to subdirectory '{subdirectory}'.");
                }
            });

            foreach (string filePath in Directory.EnumerateFiles(searchPath, fileName)) {
                if (Path.GetExtension(filePath) == fileExtension) {
                    foundFiles.Add(filePath);
                }
            }
        }
        catch (UnauthorizedAccessException) {
            // Handle unauthorized access exception for the current directory
            Console.WriteLine($"Unauthorized access to directory '{searchPath}'.");
        }

        return foundFiles;
    }

    private bool IsExcludedDirectory(string directoryPath) {
        foreach (string excludedDir in ExcludedDirectories) {
            if (directoryPath.Equals(excludedDir, StringComparison.OrdinalIgnoreCase)) {
                return true;
            }
        }

        return false;
    }
}
